/*
 Alarm System Monitor - SMS Sender
 Version: 0.1
 Author: Alex from insideGadgets (http://www.insidegadgets.com)
 Created: 15/08/2012
 Last Modified: 29/09/2012
 
 Nokia 3120 KSS (Keypad SMS Sender) which sends us an SMS when the alarm is triggered or turned off.
 
 */
 
// ATtiny261 Pin map
//
//               +-\/-+
//         PB0   1|o   |20  PA0 (ADC0)
//         PB1   2|    |19  PA1 (ADC1)
//         PB2   3|    |18  PA2 (ADC2)
//         PB3   4|    |17  PA3
//         VCC   5|    |16  AGND
//         GND   6|    |15  AVCC
//  (ADC7) PB4   7|    |14  PA4 (ADC3)
//  (ADC8) PB5   8|    |13  PA5 (ADC4)
//  (ADC9) PB6   9|    |12  PA6 (ADC5)
//   Reset PB7  10|    |11  PA7 (ADC6)
//               +----+
//
//
// Nokia 3120 Button map
// PB5 - Input from the alarm server
//
//					PB0		PB1		PB2		PB3		PB3
//					1			3			4			7			6
// PA0	2		Select	Up			Exit		Down
// PA1	5					Dial					Hangup
// PA2	9		But2					But0		But8
// PA3	10		But3		But6		Hash		But9
// PA4	8		But1		But4		Star		But7
// PA5	11														Left
// PA6	12														Right
// PA7	13														But5

#define F_CPU 1000000 // 1 MHz clock

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
#include <stdlib.h>
#include <string.h>
#include "setup.c"

volatile int alarm_on = false;
int alarm_on_last_state = false;

prog_char smsMessage_outage[] PROGMEM = "On";
prog_char smsMessage_restored[] PROGMEM = "Off";

// Break up the button number, turn on the necessary outputs, delay and then turn off
void pushButton(char buttonNumber) {
	int outputPBon = (buttonNumber % 10) - 1;
	int outputPAon = (buttonNumber / 10) - 1;
	
	PORTB |= (1<<outputPBon);
	PORTA = (1<<outputPAon);
	_delay_ms(250);
	
	PORTA = 0;
	if (outputPBon >= 0) { 
		PORTB &= ~(1<<outputPBon);
	}
	_delay_ms(250);
}

// Find which button to press for the character received
void parseChar(char c, byte onePushnumbers) {
	
	// Convert lowercase a-z to uppercase
	if (c >= 97 && c <= 122) {
		c -= 32;
	}
	
	// A-Z charset and 0-9 charset
	if ((c >= 65 && c <= 90) || (c >= 48 && c <= 57)) {
		byte lookupChar;
		byte loopEnd;
		byte buttonPressing;
		
		if (c >= 65 && c <= 90) {// A-Z charset
			lookupChar = c - 65;
			loopEnd = letterTobutton[lookupChar][buttonPushes];
			buttonPressing = letterTobutton[lookupChar][buttonTopush];
		}
		else if (c >= 48 && c <= 57) { //0-9 charset
			lookupChar = c - 48;
			loopEnd = numberTobutton[lookupChar][buttonPushes];
			buttonPressing = numberTobutton[lookupChar][buttonTopush];
		}
		
		// If we are in a section that allows numbers to be pressed just once (like dialing a number)
		if (onePushnumbers == true) {
			pushButton(buttonPressing);
		}
		else {
			for (int x = 0; x < loopEnd; x++) { // Push the corresponding button x times
				pushButton(buttonPressing);
			}
		}
	}
	
	// Dot
	else if (c == 46) {
		pushButton(button1);
	}
	
	// Space
	else if (c == 32) {
		pushButton(button0);
	}
}

// Read each char in normal strings (non-PGM).
void parseString (char *stringMsg) {
	int charAt = 0;
	char c;
	while ((c = stringMsg[charAt])) {
		parseChar(c, 0);
		charAt++;
		pushButton(buttonRight);
	}
}

// Read each char in PGM strings
void parseStringPGM (char *stringMsg) {
	int charAt = 0;
	char c;
	while ((c = (char)pgm_read_byte(&stringMsg[charAt]))) {
		parseChar(c, 0);
		charAt++;
		pushButton(buttonRight);
	}
}

// Send SMS message
void sendSMS(char *smsMsg) {
	
	// Clear previous SMS
	pushButton(buttonSelect);
	pushButton(buttonDown);
	pushButton(buttonDown);
	pushButton(buttonSelect);
	_delay_ms(500);
	
	// Exit out anything that might be up
	pushButton(buttonHangup);
	_delay_ms(500);
	
	// Create new SMS message
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	_delay_ms(500);
	
	// Parse message
	parseStringPGM(smsMsg);
	_delay_ms(500);
	
	// Select stored number
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	
	// Send
	pushButton(buttonSelect);
	
	// Wait a few seconds for message to be sent
	_delay_ms(8000);
}

// Power on/off Nokia phone
void powerPress(int button) {
	PORTB |= (1<<buttonPower);
	_delay_ms(2000);
	PORTB &= ~(1<<buttonPower);
	if (button == ON) { // Phone turned on, wait a few seconds until phone is ready
		_delay_ms(8000);
		_delay_ms(5000);
	}
	else {
		_delay_ms(2000);
	}
}

int main(void) {
	
	setup();
	
	// Set Outputs
	DDRA = 0xFF;
	DDRB |= ((1<<PB0) | (1<<PB1) | (1<<PB2) | (1<<PB3) | (1<<PB4));
	
	// Turn on pull-up resistors to save power on un-used ports
	PORTB |= ((1<<PB7) | (1<<PB6));
	
	// Pin interrupt setup
	PCMSK1 = (1<<PCINT13); // Apply interrupt to PB5 (Turns off pin change on other pins which are on by default)
	PCMSK0 = 0; // Disable PA interrupts
	sbi(GIMSK,PCIE1); // Enable pin change interrupt
	
	// Ready to send SMS
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	pushButton(buttonSelect);
	
	while(1) {
		watchdogSleep(T2S); // Keep waking up to check the state
		if (alarm_on == true && alarm_on_last_state == false) {
			sendSMS(smsMessage_outage);
			alarm_on_last_state = true;
		}
		else if (alarm_on == false && alarm_on_last_state == true) {
			sendSMS(smsMessage_restored);
			alarm_on_last_state = false;
		}
	}
}

ISR(WDT_vect) { }

ISR(PCINT_vect) {
	if (PINB & (1<<PB5)) {
		alarm_on = true;
	}
	else {
		alarm_on = false;
	}
}
